// Copyright (c) 2013 by István Váradi

// This file is part of XPLRA, a remote-access plugin for X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "MultiSetter.h"

#include "XPlane.h"

#include <xplra/Protocol.h>

#include <hu/varadiistvan/scpl/io/DataStream.h>

//------------------------------------------------------------------------------

using hu::varadiistvan::xplra::MultiSetter;

using xplra::Protocol;

using std::vector;
using std::min;

//------------------------------------------------------------------------------

MultiSetter::MultiSetter(XPlane& xplane) throw() :
    MultiBuffer(xplane, Protocol::COMMAND_REGISTER_SET_MULTI,
                Protocol::COMMAND_UNREGISTER_SET_MULTI)
{
}

//------------------------------------------------------------------------------

inline void MultiSetter::write(const DataRef& dataRef) const throw(Exception)
{
    switch (dataRef.type) {
      case Protocol::TYPE_INT:
        xplane.stream->writeS32(*reinterpret_cast<const int32_t*>
                                (data + dataRef.dataOffset));
        break;
      case Protocol::TYPE_FLOAT:
        xplane.stream->writeFloat(*reinterpret_cast<const float*>
                                (data + dataRef.dataOffset));
        break;
      case Protocol::TYPE_DOUBLE:
        xplane.stream->writeDouble(*reinterpret_cast<const double*>
                                   (data + dataRef.dataOffset));
        break;
      case Protocol::TYPE_FLOAT_ARRAY:
      case Protocol::TYPE_INT_ARRAY:
      case Protocol::TYPE_BYTE_ARRAY:
      {
          size_t elementSize =
              (dataRef.type==Protocol::TYPE_FLOAT_ARRAY) ? sizeof(float) :
              ((dataRef.type==Protocol::TYPE_INT_ARRAY) ?
               sizeof(int32_t) : sizeof(uint8_t));
          xplane.stream->write(data + dataRef.dataOffset,
                               dataRef.length * elementSize);
      }
    }
}

//------------------------------------------------------------------------------

void MultiSetter::doExecute() throw(Exception)
{
    xplane.checkStream();
    xplane.stream->writeU8(Protocol::COMMAND_EXECUTE_SET_MULTI);
    xplane.stream->writeU32(registeredID);
    for(vector<DataRef>::const_iterator i = dataRefs.begin();
        i!=dataRefs.end(); ++i)
    {
        const DataRef& dataRef = *i;
        write(dataRef);
    }
    xplane.stream->flush();
    xplane.checkResult(true);
}

//------------------------------------------------------------------------------

void MultiSetter::doExecuteUnregistered() throw(Exception)
{
    xplane.stream->writeU8(Protocol::COMMAND_SET_MULTI);
    xplane.stream->writeU32(dataRefs.size());
    for(vector<DataRef>::const_iterator i = dataRefs.begin();
        i!=dataRefs.end(); ++i)
    {
        const DataRef& dataRef = *i;
        xplane.stream->writeString(dataRef.name);
        xplane.stream->writeU8(dataRef.type);
        if (dataRef.isArray()) {
            xplane.stream->writeS32(dataRef.length);
            xplane.stream->writeS32(dataRef.offset);
        }
        write(dataRef);
    }
    xplane.stream->flush();
    xplane.checkResult(true);
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
